import { useCallback, useEffect, useState } from 'react';
/**
 * Controls the logic when an `UnreadMessagesNotification` component should be shown.
 * In virtualized message list there is no notion of being scrolled below or above `UnreadMessagesSeparator`.
 * Therefore, the `UnreadMessagesNotification` component is rendered based on timestamps.
 * If the there are unread messages in the channel and the `VirtualizedMessageList` renders
 * messages created later than the last read message in the channel, then the
 * `UnreadMessagesNotification` component is rendered. This is an approximate equivalent to being
 * scrolled below the `UnreadMessagesNotification` component.
 * @param lastRead
 * @param unreadCount
 */
export var useUnreadMessagesNotificationVirtualized = function (_a) {
    var lastRead = _a.lastRead, unreadCount = _a.unreadCount;
    var _b = useState(false), show = _b[0], setShow = _b[1];
    var toggleShowUnreadMessagesNotification = useCallback(function (renderedMessages) {
        if (!renderedMessages.length)
            return;
        var firstRenderedMessageTimestamp = renderedMessages[0].created_at
            ? new Date(renderedMessages[0].created_at).getTime()
            : 0;
        setShow(unreadCount > 0 && !!lastRead && firstRenderedMessageTimestamp > lastRead.getTime());
    }, [unreadCount, lastRead]);
    useEffect(function () {
        if (!unreadCount)
            setShow(false);
    }, [unreadCount]);
    return { show: show, toggleShowUnreadMessagesNotification: toggleShowUnreadMessagesNotification };
};
