import { useEffect } from 'react';
import { useChannelActionContext } from '../../../context';
/**
 * Takes care of marking a channel read. The channel is read only if all the following applies:
 * 1. the message list is not rendered in a thread
 * 2. the message list is scrolled to the bottom
 * 3. the channel was not marked unread by the user
 * @param isMessageListScrolledToBottom
 * @param messageListIsThread
 * @param unreadCount
 * @param wasChannelMarkedUnread
 */
export var useMarkRead = function (_a) {
    var isMessageListScrolledToBottom = _a.isMessageListScrolledToBottom, markReadOnScrolledToBottom = _a.markReadOnScrolledToBottom, messageListIsThread = _a.messageListIsThread, unreadCount = _a.unreadCount;
    var markRead = useChannelActionContext('useMarkRead').markRead;
    useEffect(function () {
        var shouldMarkRead = !messageListIsThread &&
            isMessageListScrolledToBottom &&
            markReadOnScrolledToBottom &&
            unreadCount > 0;
        var onVisibilityChange = function () {
            if (!document.hidden && shouldMarkRead)
                markRead();
        };
        document.addEventListener('visibilitychange', onVisibilityChange);
        if (shouldMarkRead)
            markRead();
        return function () {
            document.removeEventListener('visibilitychange', onVisibilityChange);
        };
    }, [
        isMessageListScrolledToBottom,
        markRead,
        messageListIsThread,
        unreadCount,
        markReadOnScrolledToBottom,
    ]);
};
